﻿public class RareLoot : Mod
{
    private HarmonyLib.Harmony harmony;
    private static string modId = "com.raftmodding.mods.rare-loot";
    private static System.Collections.Generic.Dictionary<string, System.Collections.Generic.Dictionary<string, float>> randomizerCategoryDictionary = new System.Collections.Generic.Dictionary<string, System.Collections.Generic.Dictionary<string, float>>();
    private static System.Collections.Generic.Dictionary<string, System.Collections.Generic.Dictionary<string, float>> randomDropperDictionary = new System.Collections.Generic.Dictionary<string, System.Collections.Generic.Dictionary<string, float>>();
    public void FillSpawnChances(Randomizer randomizer)
    {
        float totalWeight = randomizer.TotalWeight;
        foreach (RandomItem randomItem in randomizer.items)
        {
            if (randomItem.obj != null)
            {
                randomItem.spawnChance = randomItem.weight / totalWeight * 100f + "%";
            }
        }
    }
    public void Start()
    {
        randomizerCategoryDictionary.Clear();
        randomDropperDictionary.Clear();
        foreach (SO_RandomizerCategory randomizerCategory in UnityEngine.Resources.FindObjectsOfTypeAll<SO_RandomizerCategory>())
        {
            randomizerCategoryDictionary.Add(randomizerCategory.name, new System.Collections.Generic.Dictionary<string, float>());
            foreach (RandomItem randomItem in randomizerCategory.randomizer.items)
            {
                randomizerCategoryDictionary[randomizerCategory.name].Add(randomItem.obj?.name ?? "", randomItem.weight);
                if (randomItem.weight > 0)
                {
                    randomItem.weight = 1f;
                }
            }
            FillSpawnChances(randomizerCategory.randomizer);
        }
        foreach (SO_RandomDropper randomDropper in UnityEngine.Resources.FindObjectsOfTypeAll<SO_RandomDropper>())
        {
            if (randomDropper.name != "Dropper_FishingRod_Metal")
            {
                randomDropperDictionary.Add(randomDropper.name, new System.Collections.Generic.Dictionary<string, float>());
                foreach (RandomItem randomItem in randomDropper.randomizer.items)
                {
                    randomDropperDictionary[randomDropper.name].Add(randomItem.obj?.name ?? "", randomItem.weight);
                    if (randomItem.obj == null)
                    {
                        randomItem.weight = 0f;
                        randomItem.spawnChance = "0%";
                    }
                    else
                    {
                        if (randomItem.weight > 0)
                        {
                            randomItem.weight = 1f;
                        }
                    }
                }
                FillSpawnChances(randomDropper.randomizer);
            }
        }
        harmony = new HarmonyLib.Harmony(modId);
        harmony.PatchAll();
    }
    private void RestoreWeights(System.Collections.Generic.Dictionary<string, System.Collections.Generic.Dictionary<string, float>> dictionary, string name, Randomizer randomizer)
    {
        System.Collections.Generic.Dictionary<string, float> randomItemsDictionary = new System.Collections.Generic.Dictionary<string, float>();
        if (dictionary.TryGetValue(name, out randomItemsDictionary))
        {
            foreach (RandomItem randomItem in randomizer.items)
            {
                float weight;
                if (randomItemsDictionary.TryGetValue(randomItem.obj?.name ?? "", out weight))
                {
                    randomItem.weight = weight;
                }
            }
            FillSpawnChances(randomizer);
        }
    }
    public void OnModUnload()
    {
        harmony.UnpatchAll(modId);
        foreach (SO_RandomizerCategory randomizerCategory in UnityEngine.Resources.FindObjectsOfTypeAll<SO_RandomizerCategory>())
        {
            RestoreWeights(randomizerCategoryDictionary, randomizerCategory.name, randomizerCategory.randomizer);
        }
        foreach (SO_RandomDropper randomDropper in UnityEngine.Resources.FindObjectsOfTypeAll<SO_RandomDropper>())
        {
            if (randomDropper.name != "Dropper_FishingRod_Metal")
            {
                RestoreWeights(randomDropperDictionary, randomDropper.name, randomDropper.randomizer);
            }
        }
    }
    [HarmonyLib.HarmonyPatch(typeof(Pickup_Specific_MysteryPackage), "PickupSpecific", new System.Type[] { typeof(PlayerInventory) })]
    [HarmonyLib.HarmonyPriority(HarmonyLib.Priority.Low)]
    public class HarmonyPatch_PickupSpecificMysteryPackage_PickupSpecific
    {
        static void Prefix(ref float ___mysteryLootDropChance, ref PlayerInventory inventory)
        {
            ___mysteryLootDropChance = 100f;
        }
    }
    [HarmonyLib.HarmonyPatch(typeof(RareMaterial), "AssignRandomRareMaterial")]
    [HarmonyLib.HarmonyPriority(HarmonyLib.Priority.Low)]
    public class HarmonyPatch_RareMaterial_AssignRandomRareMaterial
    {
        static void Prefix(Randomizer ___randomizer)
        {
            foreach (RandomItem randomItem in ___randomizer.items)
            {
                if (randomItem.weight > 0)
                {
                    randomItem.weight = 1f;
                }
            }
            float totalWeight = ___randomizer.TotalWeight;
            foreach (RandomItem randomItem in ___randomizer.items)
            {
                randomItem.spawnChance = randomItem.weight / totalWeight * 100f + "%";
            }
        }
    }
}
